/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package controller;
import java.awt.event.*;

/**
 * <p>Title: AbstractTransformer</p>
 * <p>Description: Superclass representing a transformation that can be applied to a shape</p>
 * <p>Copyright: Copyright (c) 2003 Mohammed Elghaouat, Eric Paquette</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.2 $
 */
abstract public class AbstractTransformer {
	static protected int idSelector   = 1;
	static protected int idTranslator = 2;
	static protected int idTransformersIndex = 3;
	static boolean lastMousePressIn = false;

	/**
	 * Default constructor
	 */
	public AbstractTransformer() {
	}
    
	/**
	 * Transformer ID getter
	 * */
	public abstract int getID();

	/**
	 * Has mouse been clicked ?
	 * @return mouse clicked status flag
	 */
	protected boolean mouseClicked(MouseEvent e){
		return false;
	}

	/**
	 * Has mouse been released ?
	 * @return mouse released status flag
	 */
	protected boolean mouseReleased(MouseEvent e){
		return false;
	}

	/**
	 * Has mouse been pressed ?
	 * @return mouse pressed status flag
	 */
	protected boolean mousePressed(MouseEvent e){
		return false;
	}

	/**
	 * Has mouse been dragged ?
	 * @return mouse dragged status flag
	 */
	protected boolean mouseDragged(MouseEvent e){
		return false;
	}

	/**
	 * Has mouse moved ?
	 * @return mouse moved status flag
	 */
	protected boolean mouseMove(MouseEvent e){
		return false;
	}
}