/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package controller;

import model.*;
import view.Application;
import java.awt.event.*;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.awt.Point;

/**
 * <p>Title: Selector</p>
 * <p>Description: Class that selects multiple shapes using a rectangular selection area.</p>
 * <p>Copyright: Copyright (c) 2003 Mohammed Elghaouat, Eric Paquette</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.4 $
 */
public class Selector extends AbstractTransformer {
	private Point selectionOrigine;

	/**
	 * Default constructor
	 */
	public Selector() {
		selectionOrigine = new Point();
	}

	 /**
	  * Get the 
	  * @return Selector ID
	  */
	public int getID(){
		return idSelector;
	}

	/**
	 * Mouse clicked event management method
	 * @param e mouse event generated by the system
	 */
	protected boolean mouseClicked(MouseEvent e){
		// TODO EP This is not correct. We should only look at mouseReleased. 
		// If the mouse moves, then the translator will get the mouseRelease (which is ok),
		// If the mouse is not on an object on the mousePressed, we should use a
		// RectangularRegionSelector which will appropriately handle the mouseRelease.
		  
		System.out.println("mouseClicked()");
		Document doc = Application.getInstance().getActiveDocument();
		doc.clearSelectedObjectList();

		Shape shape;
		List list = doc.getDocunentObjects();
		// TODO EP Here note the use of listIterator and the backward visit of the list.
		// At the moment the selection is still incorrect with respect to objects on
		// top of others, but this is most likely due to the fact that we do too much work
		// in the mouse handlers in this classe.
		ListIterator iter = list.listIterator(list.size());
		while (iter.hasPrevious()) {
			shape = (Shape)iter.previous();

			if (shape.isThisPointIn(e.getPoint())) {
				doc.addSelectedObject(shape);
				// TODO EP This selects a single object.
				return true;
			}
		}

		return true;
	}

	/**
	 * Mouse pressed event management method
	 * @param e mouse event generated by the system
	 */
	protected boolean mousePressed(MouseEvent e){
		System.out.println("mousePressed()");
		int i = 0;
		Shape shape;
		Document doc = Application.getInstance().getActiveDocument();
		Iterator iter = doc.getDocunentObjects().listIterator();

		while (iter.hasNext()) {
			shape = (Shape)iter.next();

			if (shape.isThisPointIn(e.getPoint())) {
				if (!doc.getSelectedObjects().contains(shape)) {
					doc.clearSelectedObjectList();
					doc.addSelectedObject(shape);
				}

				lastMousePressIn = true;
				return true;
			}
		}

		selectionOrigine.setLocation(e.getX(),e.getY());
		doc.makeSelectionRectangle();
		doc.clearSelectedObjectList();
		return true;
	}

	/**
	 * Mouse dragged event management method
	 * @param e mouse event generated by the system
	 */
	protected boolean mouseDragged(MouseEvent e){
		System.out.println("mouseDragged()");
		Document doc = Application.getInstance().getActiveDocument();

		if (lastMousePressIn) {
			return false;
		} else {
			doc.getSelectionRectangle().setCorners(selectionOrigine,e.getPoint());
			doc.notifyDocChanged();
			return true;
		}
	}

	/**
	 * Mouse released event management method
	 * @param e mouse event generated by the system
	 */
	protected boolean mouseReleased(MouseEvent e) {
		System.out.println("mouseReleased()");
		Shape shape;
		Document doc = Application.getInstance().getActiveDocument();
		Rectangle selectionRectangle = doc.getSelectionRectangle();
		Iterator iter = doc.getDocunentObjects().listIterator();

		// TODO EP The selection rectangle is sometimes null... 
		//      Added the if as a quick fix to remove the Exception, 
		// has to be taken care of...
		if (selectionRectangle != null) { 
		while (iter.hasNext()) {
			shape = (Shape)iter.next();

			if (selectionRectangle.isThisPointWithIn(shape.getCenter())) {
				doc.addSelectedObject(shape);
			}
		}
		}

		doc.deleteSelectionRectangle();
		lastMousePressIn = false;
		doc.notifyDocChanged();
		return true;
	}
}