/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package controller;
import java.awt.event.MouseEvent;

/**
 * <p>Title: TransformersIndex</p>
 * <p>Description: Class that contains and returns the appropriate transformer when requested by an AbstractTransformer class instance or a child instance.</p>
 * <p>Copyright: Copyright (c) 2003 Mohammed Elghaouat, Eric Paquette</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.2 $
 */
public class TransformersIndex extends AbstractTransformer {
	private Selector theSelector;
	private Translator theTranslator;
	private AbstractTransformer TransformersArray[][];
	
	/**
	 * Default constructor
	 */
	public TransformersIndex() {
		int MaxEvtIdx;

		MaxEvtIdx = Math.max(MouseEvent.MOUSE_PRESSED,
							 Math.max(MouseEvent.MOUSE_DRAGGED,
									  MouseEvent.MOUSE_RELEASED));

		TransformersArray = new AbstractTransformer[3][MaxEvtIdx+1];
		theSelector = new Selector();
		theTranslator = new Translator();
		TransformersArray[idSelector][MouseEvent.MOUSE_DRAGGED] = theTranslator;
		TransformersArray[idTranslator][MouseEvent.MOUSE_RELEASED] = theSelector;
		TransformersArray[idTranslator][MouseEvent.MOUSE_PRESSED] = theSelector;
	}
        
	/**
	 * Transformer ID getter
	 */
	public int getID(){
		return idTransformersIndex;
	}

	/**
	 * Returns the appropriate transformer from an AbstractTransformer request
	 * @param aEvent mouse event generated by the system
	 * @param aTransformer AbstractTransformer instance that generated the event
	 * @throws TransformerUnfound
	 */
	public AbstractTransformer getTheAppropriateTransformer(MouseEvent aEvent,
		AbstractTransformer aTransformer){
		return TransformersArray[aTransformer.getID()][aEvent.getID()];
	}

	/**
	 * Returns the default transformer
	 * @return default transformer instance (the selector)
	 */
	public AbstractTransformer getTheDefaultTransformer(){
		return theSelector;
	}
}