/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package model;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

/**
 * <p>Title: ImageX</p>
 * <p>Description: Subclass of a Shape, representing an Image.</p>
 * <p>Copyright: Copyright (c) 2003 Colin Barr-Brisebois, ric Paquette</p>
 * <p>Company: ETS - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.2 $
 */
public class ImageX extends Shape implements ShapeObservable {
    private String filename = "";                      
   	private int width = 0;
   	private int height = 0;
   	
    private Image imgOriginal = null;
    private Image imgCurrent = null;
    private Image imgFiltered = null;    

    private Pixel[][] pixels = null;
    private PixelGrabber pixelGrabber = null;
    private Toolkit toolkit = Toolkit.getDefaultToolkit();
    
    /**
     * Creates an ImageX class with an image file
     */
    public ImageX(int centerX, int centerY, String filename) {
        this(centerX, centerY, filename, null);       
    }

    /**
     * Creates an ImageX class with an image file
     */
    public ImageX(int centerX, int centerY, String filename, ImageFilter filter) {
        super(centerX, centerY);
        loadImage(filename, filter);
    }    
    
    /**
     * Loads an image from a specified file
     */
    public void loadImage(String filename, ImageFilter filter) {     
        ImageIcon tmpIcon = null;
        this.filename = filename; 

        imgOriginal = toolkit.createImage(this.filename);
        imgFiltered = toolkit.createImage(this.filename);
        
        if (filter != null) {
            imgFiltered = toolkit.createImage(new FilteredImageSource(imgFiltered.getSource(), filter));
            imgCurrent = imgFiltered;
        }
       
        imgCurrent = imgOriginal;
        
        tmpIcon = new ImageIcon(imgOriginal);
        width = tmpIcon.getIconWidth();
		height = tmpIcon.getIconHeight();
                
        loadPixelGrabber();        
    }     
     
    public void setFilter(ImageFilter filter) {
        imgFiltered = toolkit.createImage(new FilteredImageSource(imgFiltered.getSource(), filter));
        imgCurrent = imgFiltered;
        loadPixelGrabber();
    }
    
/*	Removed because translator performs the transformations... ?
 	public void rotate(int angleDegres) {
		//matTransformation.rotate(Math.toRadians(angleDegres));
	}
	
	public void scale(double x, double y) {
		//matTransformation.scale(x, y);
	}
	
	public void shear(double x, double y) {
		//matTransformation.shear(x, y);
	}

	public void translate(int x, int y) {
		//matTransformation.translate(x, y);
	}
*/	
	/**
	 * Returns the pixel at the specified coordinate
	 */
	public Pixel getPixel(int x, int y) {
		return pixels[x][y];
	}
	
	/*
	 * TODO Find if point is in shape after all transforms...( ask Eric )
	 * How can we fit this in if all the transforms are made in the controller...
	 */
	public boolean isThisPointIn(Point aPt) {
		int distFromCenterX = (int)Math.abs(aPt.getX()-getCenter().getX());
		int distFromCenterY = (int)Math.abs(aPt.getY()-getCenter().getY());
		boolean onX = (distFromCenterX <= width / 2);
		boolean onY = (distFromCenterY <= height / 2);

		if (onX && onY) {
		  return true;
		} else{
		  return false;
		}
	}

    /**
     * Loads the pixel grabber and stores the pixel data in the pixel array
     * TODO Better error messages
     * TODO Call when size changed for new pixel coords.
     */ 
    private void loadPixelGrabber() {
        int[] pixels = new int[width * height];
            
        try {
            pixelGrabber = new PixelGrabber(imgOriginal,
                                            0, 
                                            0, 
                                            width,
                                            height,
                                            pixels,
                                            0,
                                            width);
        
            pixelGrabber.grabPixels();
            
            if ((pixelGrabber.getStatus() & ImageObserver.ABORT) != 0) {
                System.err.println("Image fetch aborded or errored.");
            } else {
                processPixels(pixels, width, height);
            }
            
		} catch (InterruptedException e) {
	   		System.err.println("Interrupted while initializing PixelGrabber.");
		}
    }    
   
   /**
    * Draws the image 
    */
    public void draw(Graphics gfx) {
        gfx.drawImage(imgCurrent, 
        			  (int)(getCenter().getX()-width/2),
        			  (int)(getCenter().getY()-height/2), 
        			  width, 
        			  height, 
        			  null);
    }
         
    /**
     * Pixel array conversion processing code
     */ 
    private void processPixels(int[] pixels, int width, int height) {
        this.pixels = new Pixel[width][height];
        
        for (int y = 0; y < height; y++) {
            for (int x = 0; x < width; x++) {
                this.pixels[x][y] = new Pixel(pixels[y * width + x]);
            }
        }
    }
}