/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package model;
import java.awt.Graphics;
import java.awt.Point;

/**
 * <p>Title: Rectangle</p>
 * <p>Description: Subclass of a Shape, representing a Rectangle.</p>
 * <p>Copyright: Copyright (c) 2004 Mohammed Elghaouat, Eric Paquette, Colin B-B</p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.2 $
 */
public class Rectangle extends ColoredShape {
 	private int width;
 	private int height;

	/**
	 * Constructor where rectangles attributes can be specified.
	 * @param aCenterX x center coordinate of the rectangle
	 * @param aCenterY y center coordinate of the rectangle
	 * @param aWidth width value of the rectangle
	 * @param aHeight height value of the rectangle
	 */
	public Rectangle(int centerX, int centerY, int width, int height) {
   		super(centerX, centerY);
   		this.width = width;
   		this.height = height;
        getColor().setRed(255);
  	}
  	
 	/**
 	 * Rectangle rendering method.
 	 * @param g Graphics rendering context
 	 */
 	public void draw(Graphics g){
 		g.setColor(getColor().toColor());
   		g.drawRect((int)(getCenter().getX()-width/2),
   				   (int)(getCenter().getY()-height/2),
   				   width,
   				   height);
 	}

	/**
	 * Is the specified point within the rectangle selection thickness?
	 * @param aPt specified point coordinates
	 * @return point contained inside flag value
	 */
 	public boolean isThisPointIn(Point aPt) {
   		int distX =(int) Math.abs(aPt.getX()-getCenter().getX());
   		int distY =(int) Math.abs(aPt.getY()-getCenter().getY());
   		boolean onX = (Math.abs(distY-height/2)<=SELECTION_THICKNESS)&&(distX<=width/2);
   		boolean onY = (Math.abs(distX-width/2)<=SELECTION_THICKNESS)&&(distY<=height/2);

   		if (onX||onY) {
     		return true;
   		} else {
     		return false;
   		}
 	}

	/**
	 * Is the specified point within the rectangle area?
	 * @param aPt specified point coordinates
	 * @return point contained inside flag value
	 */
 	public boolean isThisPointWithIn(Point aPt){
  		int distX =(int) Math.abs(aPt.getX()-getCenter().getX());
  		int distY =(int) Math.abs(aPt.getY()-getCenter().getY());

  		if ((distX<width/2)&&(distY<height/2)) {
    		return true;
  		} else {
    		return false;
  		}
 	}

	/**
	 * Set the rectangle corner coordinates
	 * @param aCorner1 first corner coordinate (top-left)
	 * @param aCorner2 second corner coordinate (bottom-right)
	 */
 	public void setCorners(Point aCorner1, Point aCorner2){
   		double maxX = Math.max(aCorner2.getX(),aCorner1.getX());
   		double maxY = Math.max(aCorner2.getY(),aCorner1.getY());

		setCenter(maxX - Math.abs(aCorner2.getX()-aCorner1.getX())/2,
				  maxY - Math.abs(aCorner2.getY()-aCorner1.getY())/2);

   		width  = (int)Math.abs(aCorner2.getX()-aCorner1.getX());
   		height = (int)Math.abs(aCorner2.getY()-aCorner1.getY());
 	}
}