/*
   This file is part of j2dcg.
   j2dcg is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
   j2dcg is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   You should have received a copy of the GNU General Public License
   along with j2dcg; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

package view;
import model.*;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.*;
import java.util.Iterator;
import java.util.List;

import javax.swing.*;
import javax.swing.border.EtchedBorder;

/**
 * <p>Title: Application</p>
 * <p>Description: Class managing the runtime application instance</p>
 * <p>Copyright: Copyright (c) 2003 **fill me** </p>
 * <p>Company: (TS) - cole de Technologie Suprieure</p>
 * @author unascribed
 * @version $Revision: 1.3 $
 */
public class Application extends JFrame {
  
  	private Window window;
  	private Menu menu;
  	private Document doc;
  	
  	static Application theInstance;
  	
	/**
	 * Default constructor.
	 */
  	public Application() {}
  	
	/**
	 * Get the active document.
	 * @return active document instance
	 */
	public Document getActiveDocument(){
		return doc;
	}  	

	/**
	 * Get the current instance of the application.
	 * @return current instance of the application
	 */
  	static public Application getInstance(){
		return theInstance;
  	}
  	
  	/**
  	 * Launches the application
  	 */
  	private void launchApplication(){
  		doc = new Document();
    	window = new Window();
	    menu = new Menu();
	    JPanel objectsPanel = window;

		JPanel parametersPanel = new JPanel();
		parametersPanel.setLayout(new FlowLayout(FlowLayout.LEADING));
		parametersPanel.setBorder(BorderFactory.createEtchedBorder(EtchedBorder.LOWERED));
		ColorParameter colorParameter = new ColorParameter();
		parametersPanel.add(colorParameter);
		doc.addObserver(colorParameter);
		
		getContentPane().add(parametersPanel, BorderLayout.NORTH);
	    getContentPane().add(window, BorderLayout.CENTER);
	    setJMenuBar(menu);
	    setSize(400,400);
	    setVisible(true);
    	
    	addWindowListener(new WindowAdapter() { 
	                          public void windowClosing(WindowEvent e) {
	                              System.exit(0);
	                          }
                          });
  	}
  	
	public void colorSelectedObjects(Pixel p) {
		if (p == null)
			return;
			
		// TODO This application logic should go in the controller or the model.
		List list = theInstance.getActiveDocument().getSelectedObjects();
		Iterator i = list.iterator();
		while (i.hasNext()) {
			Shape s = (Shape)i.next();
			if (s instanceof ColoredShape) {
				ColoredShape cs = (ColoredShape)s;
				cs.setColor(p);
			}
		}
	}

  	/**
  	 * MAIN
  	 * @param args main's parameters
  	 */
  	public static void main(String[] args) {
    	theInstance = new Application();
    	theInstance.launchApplication();
  	}
}